@@ -69,6 +69,21 @@ sub new_from_description
 
 sub shutdown { }
 
+sub make_protocol_GPIO
+{
+    my $self = shift;
+
+    require Device::FTDI::MPSSE;
+
+    my $mpsse = Device::Chip::Adapter::FTDI::_base->new(
+        Device::FTDI::MPSSE->new( ftdi => $self->{ftdi} ),
+    );
+
+    $self->{protocol} = $mpsse;
+
+    Future->done( $mpsse );
+}
+
 sub make_protocol_SPI
 {
     my $self = shift;
@@ -95,8 +110,118 @@ sub make_protocol_I2C
     Future->done( $i2c );
 }
 
+package
+    Device::Chip::Adapter::FTDI::_base;
+
+use Carp;
+
+use Device::FTDI::MPSSE qw( DBUS CBUS );
+
+sub new
+{
+    my $class = shift;
+    my ( $mpsse ) = @_;
+
+    return bless {
+        mpsse => $mpsse,
+    }, $class;
+}
+
+# Basic FTDI has no control of power
+sub power { Future->done }
+
+sub list_gpios
+{
+    return ( map { "D$_" } 0 .. 7 ),
+           ( map { "C$_" } 0 .. 7 );
+}
+
+sub write_gpios
+{
+    my $self = shift;
+    my ( $gpios ) = @_;
+
+    my %val;
+    my %mask;
+
+    foreach my $gpio ( keys %$gpios ) {
+        my ( $bus, $num ) = $gpio =~ m/^([DC])([0-7])$/ or
+            croak "Unrecognised GPIO name $gpio";
+
+        my $bit = 1 << $num;
+
+        $val {$bus} |= $gpios->{$gpio} ? $bit : 0;
+        $mask{$bus} |= $bit;
+    }
+
+    my @f;
+
+    push @f, $self->{mpsse}->write_gpio( DBUS, $val{D}, $mask{D} ) if $mask{D};
+    push @f, $self->{mpsse}->write_gpio( CBUS, $val{C}, $mask{C} ) if $mask{C};
+
+    Future->needs_all( @f );
+}
+
+sub read_gpios
+{
+    my $self = shift;
+    my ( $gpios ) = @_;
+
+    my %mask = ( D => 0, C => 0 );
+
+    foreach my $gpio ( @$gpios ) {
+        my ( $bus, $num ) = $gpio =~ m/^([DC])([0-7])$/ or
+            croak "Unrecognised GPIO name $gpio";
+
+        my $bit = 1 << $num;
+
+        $mask{$bus} |= $bit;
+    }
+
+    Future->needs_all(
+        $mask{D} ? $self->{mpsse}->read_gpio( DBUS, $mask{D} ) : Future->done(0),
+        $mask{C} ? $self->{mpsse}->read_gpio( CBUS, $mask{C} ) : Future->done(0),
+    )->then( sub {
+        my ( $dbus, $cbus ) = @_;
+
+        my %vals;
+
+        foreach my $num ( 0 .. 7 ) {
+            my $bit = 1 << $num;
+
+            $vals{"D$num"} = !!( $dbus & $bit ) if $mask{D} & $bit;
+            $vals{"C$num"} = !!( $cbus & $bit ) if $mask{C} & $bit;
+        }
+
+        Future->done( \%vals );
+    });
+}
+
+sub tris_gpios
+{
+    my $self = shift;
+    my ( $gpios ) = @_;
+
+    my %mask = ( D => 0, C => 0 );
+
+    foreach my $gpio ( @$gpios ) {
+        my ( $bus, $num ) = $gpio =~ m/^([DC])([0-7])$/ or
+            croak "Unrecognised GPIO name $gpio";
+
+        my $bit = 1 << $num;
+
+        $mask{$bus} |= $bit;
+    }
+
+    Future->needs_all(
+        $mask{D} ? $self->{mpsse}->tris_gpio( DBUS, $mask{D} ) : Future->done(0),
+        $mask{C} ? $self->{mpsse}->tris_gpio( CBUS, $mask{C} ) : Future->done(0),
+    )->then_done();
+}
+
 package
     Device::Chip::Adapter::FTDI::_SPI;
+use base qw( Device::Chip::Adapter::FTDI::_base );
 
 use Carp;
 
@@ -105,10 +230,7 @@ sub new
     my $class = shift;
 
     require Device::FTDI::SPI;
-
-    return bless {
-        spi => Device::FTDI::SPI->new( @_ ),
-    }, $class;
+    return $class->SUPER::new( Device::FTDI::SPI->new( @_ ) );
 }
 
 sub configure
@@ -122,21 +244,19 @@ sub configure
     croak "Unrecognised configuration options: " . join( ", ", keys %args )
         if %args;
 
-    my $spi = $self->{spi};
+    my $spi = $self->{mpsse};
     $spi->set_spi_mode( $mode )          if defined $mode;
     $spi->set_clock_rate( $max_bitrate ) if defined $max_bitrate;
 
     Future->done;
 }
 
-# Basic FTDI has no control of power
-sub power { Future->done }
-
-sub write     { shift->{spi}->write( @_ ) }
-sub readwrite { shift->{spi}->readwrite( @_ ) }
+sub write     { $_[0]->{mpsse}->write( @_ ) }
+sub readwrite { $_[0]->{mpsse}->readwrite( @_ ) }
 
 package
     Device::Chip::Adapter::FTDI::_I2C;
+use base qw( Device::Chip::Adapter::FTDI::_base );
 
 use Carp;
 
@@ -145,10 +265,7 @@ sub new
     my $class = shift;
 
     require Device::FTDI::I2C;
-
-    return bless {
-        i2c => Device::FTDI::I2C->new( @_ ),
-    }, $class;
+    return $class->SUPER::new( Device::FTDI::I2C->new( @_ ) );
 }
 
 # TODO - addr ought to be a mount option somehow
@@ -164,24 +281,21 @@ sub configure
         if %args;
 
     $self->{addr} = $addr if defined $addr;
-    $self->{i2c}->set_clock_rate( $max_bitrate ) if defined $max_bitrate;
+    $self->{mpsse}->set_clock_rate( $max_bitrate ) if defined $max_bitrate;
 
     Future->done;
 }
 
-# Basic FTDI has no control of power
-sub power { Future->done }
-
 sub write
 {
     my $self = shift;
-    $self->{i2c}->write( $self->{addr}, @_ );
+    $self->{mpsse}->write( $self->{addr}, @_ );
 }
 
 sub write_then_read
 {
     my $self = shift;
-    $self->{i2c}->write_then_read( $self->{addr}, @_ );
+    $self->{mpsse}->write_then_read( $self->{addr}, @_ );
 }
 
 =head1 AUTHOR
